@@ -14,20 +14,18 @@ import threading
 import time
 import unicodedata
 
-__version__ = '0.0.10'
+__version__ = '0.1.0'
 
 log = logging.getLogger(__name__)
 #log=logging
 
 log.debug('version=' + __version__)
 
-# deadlock test fails if USE_FILES=0
-USE_FILES = 1
 
 CONFIG_FILE = '.easyprocess.cfg'
 SECTION_LINK = 'link'
 POLL_TIME = 0.1
-USE_POLL = 1
+USE_POLL = 0
     
 class EasyProcessError(Exception):
     """  
@@ -69,11 +67,19 @@ class Proc():
     '''
     config = None
     
-    def __init__(self, cmd, ubuntu_package=None, url=None, max_bytes_to_log=1000, cwd=None):
+    def __init__(self, cmd, ubuntu_package=None, url=None, max_bytes_to_log=1000, cwd=None, use_temp_files=True):
         '''
         :param cmd: string ('ls -l') or list of strings (['ls','-l']) 
-        :param max_bytes_to_log: logging of stdout and stderr is limited by this value
+        :param max_bytes_to_log: logging of stdout and stderr is limited 
+                                 by this value
+        :param use_temp_files: use temp files instead of pipes for 
+                               stdout and stderr,
+                               pipes can cause deadlock in some cases
+                               (see unit tests)
         '''
+        self.use_temp_files = use_temp_files
+        self._outputs_processed = False
+
         self.popen = None
         self.stdout = None
         self.stderr = None
@@ -227,9 +233,9 @@ class Proc():
         if self.is_started:
             raise EasyProcessError(self, 'process was started twice!')
 
-        if USE_FILES:
-            self._stdout_file = tempfile.NamedTemporaryFile(prefix='stdout_')
-            self._stderr_file = tempfile.NamedTemporaryFile(prefix='stderr_')
+        if self.use_temp_files:
+            self._stdout_file = tempfile.TemporaryFile(prefix='stdout_')
+            self._stderr_file = tempfile.TemporaryFile(prefix='stderr_')
             stdout = self._stdout_file
             stderr = self._stderr_file
             
@@ -251,20 +257,17 @@ class Proc():
         self.is_started = True
         log.debug('process was started (pid=%s)' % (str(self.pid),))
 
-        def target():
-#            log.debug('Thread started')        
-            self._wait4process()
-#            log.debug('Thread finished')
+#        def target():
+#            self._wait4process()
             
-        def shutdown():
-#            log.debug('stopping thread')
-            self._stop_thread = True
-            self._thread.join()
+#        def shutdown():
+#            self._stop_thread = True
+#            self._thread.join()
         
-        self._thread = threading.Thread(target=target)
-        self._thread.daemon = 1
-        self._thread.start()
-        atexit.register(shutdown)
+#        self._thread = threading.Thread(target=target)
+#        self._thread.daemon = 1
+#        self._thread.start()
+#        atexit.register(shutdown)
         
         return self
 
@@ -286,19 +289,26 @@ class Proc():
         
         :rtype: self
         '''
-
+            
+        if timeout is not None:
+            if not self._thread:
+                self._thread = threading.Thread(target=self._wait4process)
+                self._thread.daemon = 1
+                self._thread.start()
+                
         if self._thread:
             self._thread.join(timeout=timeout)
-            if timeout is not None:
-                self.timeout_happened = self._thread.isAlive()
+            self.timeout_happened = self.timeout_happened or self._thread.isAlive()
+        else:
+            # no timeout and no existing thread
+            self._wait4process()
+
         return self
-    
-    #def __del__(self):
-    #    if self._thread:
-    #        log.debug('waiting for thread')
-    #        self._thread.join()
-            
+                
     def _wait4process(self):
+        if self._outputs_processed:
+            return
+
         def remove_ending_lf(s):
             if s.endswith('\n'):
                 return s[:-1]
@@ -306,7 +316,7 @@ class Proc():
                 return s
                    
         if self.popen:
-            if USE_FILES:    
+            if self.use_temp_files:    
                 if USE_POLL:    
                     while 1:
                         if self.popen.poll() is not None:
@@ -316,9 +326,10 @@ class Proc():
                         time.sleep(POLL_TIME)
 
                 else:
-                    # wait() blocks process
+                    # wait() blocks process, timeout not possible
                     self.popen.wait()
                 
+                self._outputs_processed = True
                 self._stdout_file.seek(0)            
                 self._stderr_file.seek(0)            
                 self.stdout = self._stdout_file.read()
@@ -334,6 +345,9 @@ class Proc():
                 #self.popen.wait()
                 #self.stdout = self.popen.stdout.read()
                 #self.stderr = self.popen.stderr.read()
+                
+                # communicate() blocks process, timeout not possible
+                self._outputs_processed = True
                 (self.stdout, self.stderr) = self.popen.communicate()
             log.debug('process has ended')
             self.stdout = remove_ending_lf(self.stdout)
@@ -347,8 +361,6 @@ class Proc():
                 return s
             log.debug('stdout=' + limit_str(self.stdout))
             log.debug('stderr=' + limit_str(self.stderr))
-        #self.is_started = False
-        #return self
             
     def stop(self):
         '''
